#ifndef CDEPFILEDIALOG_H
#define CDEPFILEDIALOG_H
#include <gtkmm.h>
#include "CGtkPkgList.h"

// Presents a dialog allowing the user to load dependencies from a file.
// See COPYING for license information.

class CDepFileDialog : public Gtk::Dialog
{
public:
	CDepFileDialog(packagemap_t& packages);
};

CDepFileDialog::CDepFileDialog(packagemap_t& packages)
{
	set_title("Summary");
	add_button(Gtk::Stock::CLOSE, -4);
	set_border_width(6);

	// Let the user choose a file.
	Gtk::FileChooserDialog	_fdlg("Choose a file", Gtk::FILE_CHOOSER_ACTION_OPEN);
	_fdlg.set_transient_for(*this);
	_fdlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	_fdlg.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
	Gtk::FileFilter	_filter;
	_filter.set_name("Dependency Files (*.dep)");
	_filter.add_pattern("*.dep");
	_fdlg.add_filter(_filter);

	if ( _fdlg.run() == Gtk::RESPONSE_OK ) {
		// Read the deps from the file.
		_fdlg.hide();
		std::ifstream	_ifdep;
		_ifdep.open(_fdlg.get_filename().c_str(), std::ios_base::in);
		std::string	_depnames;
		getline(_ifdep, _depnames);

		if ( _depnames.length() ) _depnames += ",";
		else	get_vbox()->pack_start(*(new Gtk::Label("Empty dependency file")));

		// Process the deps.
		while ( (gint32)_depnames.find(',') >= 0 ) {
			Glib::ustring	_name = _depnames.substr(0, _depnames.find(','));
			if ( packages[_name].color != COLOR_BLACKLIST && !packages[_name].installedversion.length() && packages[_name].mirrorversion.length() )
				packages[_name].color = COLOR_INSTALL;	// Mark the package for installation.

			_depnames = _depnames.substr(_depnames.find(',') + 1);
			if ( packages[_name].installedversion.length() )
				get_vbox()->pack_start(*(new Gtk::Label("Already installed:\t\t" + _name, Gtk::ALIGN_LEFT)));
			else if ( !packages[_name].mirrorversion.length() )
				get_vbox()->pack_start(*(new Gtk::Label("Not found on mirror:\t" + _name, Gtk::ALIGN_LEFT)));
			else	get_vbox()->pack_start(*(new Gtk::Label("To be installed:\t\t" + _name, Gtk::ALIGN_LEFT)));
			if ( !packages[_name].installedversion.length() && !packages[_name].mirrorversion.length() )
				packages.erase(_name);	// This package doesn't exist, so clear out what we did to it.
		}

		show_all_children();
		run();
	}
}

#endif
