#ifndef CDOTNEWDIALOG_H
#define CDOTNEWDIALOG_H

#include <gtkmm.h>
#include <vector>
#include <fstream>
#include "CGtkTextView.h"

// This class presents a dialog to ask the user what exactly he wants to
// do with new configuration files in /etc.
// See COPYING for license information.

class CDotnewDialog : public Gtk::Dialog
{
public:
	CDotnewDialog(std::vector<struct_package> installs);
protected:
	void	on_button_skip();
	void	on_button_removenew();
	void	on_button_overwrite();
	void	on_button_diff();
	void	shownext();

	Gtk::Label		_TextLbl;
	Gtk::Button		_SkipBut;
	Gtk::Button		_RemoveNewBut;
	Gtk::Button		_OverwriteBut;
	Gtk::Button		_DiffBut;
	Gtk::CheckButton	_BackupChk;	// Create .old files?

	enum enum_dotnewstate {
		EDOTNEW_NONE,
		EDOTNEW_OVERWRITTEN,
		EDOTNEW_REMOVED
	};
	struct struct_dotnew
	{
		Glib::ustring		name;
		enum_dotnewstate	state;
	};

	std::vector<struct_dotnew>	_DotnewFiles;
	int				_CurFile;
};

CDotnewDialog::CDotnewDialog(std::vector<struct_package> installs) :
_SkipBut("_Skip", true),
_RemoveNewBut("_Remove .new", true),
_OverwriteBut("_Overwrite", true),
_DiffBut("_View diff", true),
_BackupChk("_Backup overwritten file", true),
_CurFile(-1)
{
	// Set up the UI.
	set_title("Update configuration files");
	set_border_width(6);
	get_vbox()->set_spacing(6);
	get_vbox()->pack_start(_TextLbl);
	get_vbox()->pack_start(_BackupChk);
	_BackupChk.set_tooltip_text("Instead of overwriting the old configuration file, move\nit to {name}.old, so that you can recover it if need be");
	get_action_area()->pack_start(_SkipBut);
	_SkipBut.signal_clicked().connect(sigc::mem_fun(*this, &CDotnewDialog::on_button_skip));
	_SkipBut.set_tooltip_text("Skip this file for now, no changes will be made to it.\nYou will be prompted again next time you apply changes");
	get_action_area()->pack_start(_RemoveNewBut);
	_RemoveNewBut.signal_clicked().connect(sigc::mem_fun(*this, &CDotnewDialog::on_button_removenew));
	_RemoveNewBut.set_tooltip_text("Remove the new configuration file");
	get_action_area()->pack_start(_OverwriteBut);
	_OverwriteBut.signal_clicked().connect(sigc::mem_fun(*this, &CDotnewDialog::on_button_overwrite));
	_OverwriteBut.set_tooltip_text("Overwrite the old configuration file with the new one");
	get_action_area()->pack_start(_DiffBut);
	_DiffBut.signal_clicked().connect(sigc::mem_fun(*this, &CDotnewDialog::on_button_diff));
	_DiffBut.set_tooltip_text("Show the difference between the .new file and the old file");

	// Find the dotnew files.
	system(Glib::ustring("find /etc -name *.new > " + TMPFILE("zendo-dotnew.tmp")).c_str());
	std::ifstream	_iflist;
	std::string	_input;
	_iflist.open(TMPFILE("zendo-dotnew.tmp").c_str(), std::ios_base::in);
	while ( !_iflist.eof() && _iflist.is_open() ) {
		getline(_iflist, _input);
		struct_dotnew	_tmp;
		_tmp.name = _input;
		_tmp.state = EDOTNEW_NONE;
		if ( _input.length() ) _DotnewFiles.push_back(_tmp);
	}
	_iflist.close();

	// Run the thing if there are any dotnew files.
	if ( _DotnewFiles.size() ) {
		shownext();
		show_all_children();
		run();
	}
	hide();
}

void CDotnewDialog::on_button_skip()
{	// Do not overwrite the file, just go on.
	shownext();
}

void CDotnewDialog::on_button_removenew()
{	// Remove the .new file.
	WRITELOG("Removing dotnew file: " + _DotnewFiles[_CurFile].name);
	std::string	_cmd = "rm -f ";
	_cmd += _DotnewFiles[_CurFile].name;
	system(_cmd.c_str());
	_DotnewFiles[_CurFile].state = EDOTNEW_REMOVED;
	shownext();
}

void CDotnewDialog::on_button_overwrite()
{	// Overwrite the old file with the .new file.
	// First we need to make sure the .new file is executable if it should be.
	WRITELOG("Applying dotnew file: " + _DotnewFiles[_CurFile].name);
	std::string	_cmd = "if [ -x ";
	_cmd += _DotnewFiles[_CurFile].name.substr(0, _DotnewFiles[_CurFile].name.length() - 4);
	_cmd += " ]; then chmod +x " + _DotnewFiles[_CurFile].name;
	_cmd += "; else chmod -x " + _DotnewFiles[_CurFile].name + "; fi";
	system(_cmd.c_str());	// chmod +x it if required.
	if ( _BackupChk.get_active() ) {	// Back it up...beep...beep...beep.
		_cmd = "mv -f " + _DotnewFiles[_CurFile].name.substr(0, _DotnewFiles[_CurFile].name.length() - 4);
		_cmd += " " + _DotnewFiles[_CurFile].name.substr(0, _DotnewFiles[_CurFile].name.length() - 4) + ".old";
		system(_cmd.c_str());
	}
	_cmd = "mv -f " + _DotnewFiles[_CurFile].name + " ";
	_cmd += _DotnewFiles[_CurFile].name.substr(0, _DotnewFiles[_CurFile].name.length() - 4);
	system(_cmd.c_str());
	_DotnewFiles[_CurFile].state = EDOTNEW_OVERWRITTEN;
	shownext();
}

void CDotnewDialog::on_button_diff()
{	// Show the difference between two files.
	Gtk::Dialog	_dlg;
	CGtkTextView	_txt;
	_dlg.get_vbox()->pack_start(_txt);
	_dlg.add_button(Gtk::Stock::CLOSE, 1);
	_dlg.set_default_size(500, 400);
	_dlg.set_default_response(1);
	system(Glib::ustring("diff -ruN " +
		_DotnewFiles[_CurFile].name.substr(0, _DotnewFiles[_CurFile].name.length() - 4) +
		" " + _DotnewFiles[_CurFile].name + " > " + TMPFILE("zendo-diff.tmp")).c_str());
	_txt.GetTextFromFile(TMPFILE("zendo-diff.tmp"));
	system(Glib::ustring("rm " + TMPFILE("zendo-diff.tmp")).c_str());
	_dlg.show_all_children();
	_dlg.run();
}

void CDotnewDialog::shownext()
{	// Sets up the dialog for the next dotnew file.
	std::ifstream	_iftmp;
	_CurFile++;
	if ( _CurFile == _DotnewFiles.size() ) {
		bool	_sumwrite = false;
		for ( int i = 0; i < _DotnewFiles.size(); i++ )
			if ( _DotnewFiles[i].state == EDOTNEW_OVERWRITTEN ) _sumwrite = true;
		bool	_sumremove = false;
		for ( int i = 0; i < _DotnewFiles.size(); i++ )
			if ( _DotnewFiles[i].state == EDOTNEW_REMOVED ) _sumremove = true;
		if ( !_sumwrite && !_sumremove ) {
			response(1);
			return;
		}
		Glib::ustring	_summary;
		if ( _sumwrite ) {
			_summary = "The following files were overwritten:";
			for ( int i = 0; i < _DotnewFiles.size(); i++ ) {
				if ( _DotnewFiles[i].state == EDOTNEW_OVERWRITTEN )
					_summary += "\n\t" + _DotnewFiles[i].name.substr(0, _DotnewFiles[i].name.length() - 4);
			}
		}
		if ( _sumremove ) {
			_summary += "\nThe following files were removed:";
			for ( int i = 0; i < _DotnewFiles.size(); i++ ) {
				if ( _DotnewFiles[i].state == EDOTNEW_REMOVED )
					_summary += "\n\t" + _DotnewFiles[i].name;
			}
		}
		_TextLbl.set_text(_summary);
		_SkipBut.set_label("_Finish");
		_SkipBut.set_tooltip_text("Close this dialog");
		_RemoveNewBut.set_sensitive(false);
		_OverwriteBut.set_sensitive(false);
		_DiffBut.set_sensitive(false);
		_BackupChk.hide();
		return;
	} else if ( _CurFile > _DotnewFiles.size() ) {
		response(1);
		return;
	}

	std::string	_text = "Overwrite the file:\n\t";
	_text += _DotnewFiles[_CurFile].name.substr(0, _DotnewFiles[_CurFile].name.length() - 4);
	_text += "\nwith this one?\n\t";
	_text += _DotnewFiles[_CurFile].name;
	_TextLbl.set_text(_text);
	_TextLbl.modify_fg(Gtk::STATE_NORMAL, Gdk::Color("Red"));
}

#endif
